#!/usr/bin/env python

from omics_pipe.parameters.default_parameters import default_parameters
from omics_pipe.utils import *
p = Bunch(default_parameters)


def GATK_snpcal_unifiedGenotyper(sample, GATK_snpcal_unifiedGenotyper_flag):
    '''GATK_variant_discovery with UnifiedGenotyper.

    input:
        recal.FULL.bam
    output:
        snpcal_unifiedGenotyper_mbq_${9}_mmq_${4}.vcf
    citation:
        McKenna A, Hanna M, Banks E, Sivachenko A, Cibulskis K, Kernytsky A, Garimella K, Altshuler D, Gabriel S, Daly M, DePristo MA (2010). The Genome Analysis Toolkit: a MapReduce framework for analyzing next-generation DNA sequencing data. Genome Res. 20:1297-303.
    link: GATK_variant_discovery
        http://www.broadinstitute.org/gatk/
    parameters from parameters file:
        BWA_RESULTS:

        SAMPLE:

        TEMP_DIR:

        MMQ:

        GENOME:

        DBSNP:

        TARGET_FLANKS_FILE:

        GATK_THREADS:

        MBQ:

        '''

    spawn_job(jobname = 'GATK_snpcal_unifiedGenotyper', SAMPLE = sample, LOG_PATH = p.LOG_PATH, RESULTS_EMAIL = p.RESULTS_EMAIL, SCHEDULER = p.SCHEDULER, walltime = "240:00:00", queue = p.QUEUE, nodes = 1, ppn = 4, memory = "15gb", script = "/GATK_snpcal_unifiedGenotyper_DRAW.sh", args_list = [p.BWA_RESULTS, sample, p.TEMP_DIR, p.MMQ, p.GENOME, p.DBSNP, p.TARGET_FLANKS_FILE, p.GATK_THREADS, p.MBQ])
    job_status(jobname = 'GATK_snpcal_unifiedGenotyper', resultspath = p.BWA_RESULTS, SAMPLE = sample,  outputfilename = sample + "/vcf/" +  sample + "_snpcal_unifiedGenotyper_mbq_" + p.MBQ + "_mmq_" + p.MMQ +".vcf", FLAG_PATH = p.FLAG_PATH)
    return

def GATK_variantFiltrationNoMask(sample, GATK_variantFiltrationNoMask_flag):
    '''GATK_variant_filtering.

    input:
        snpcal_unifiedGenotyper_mbq_${9}_mmq_${4}.vcf
    output:
        snpcal_unifiedGenotyper_mbq_${9}_mmq_${4}_filtered_nomask.vcf
    citation:
        McKenna A, Hanna M, Banks E, Sivachenko A, Cibulskis K, Kernytsky A, Garimella K, Altshuler D, Gabriel S, Daly M, DePristo MA (2010). The Genome Analysis Toolkit: a MapReduce framework for analyzing next-generation DNA sequencing data. Genome Res. 20:1297-303.
    link: GATK_variant_discovery
        http://www.broadinstitute.org/gatk/
    parameters from parameters file:
        BWA_RESULTS:

        SAMPLE:

        TEMP_DIR:

        MMQ:

        GENOME:

        DBSNP:

        TARGET_FLANKS_FILE:

        GATK_THREADS:

        MBQ:

        '''

    spawn_job(jobname = 'GATK_variantFiltrationNoMask', SAMPLE = sample, LOG_PATH = p.LOG_PATH, RESULTS_EMAIL = p.RESULTS_EMAIL, SCHEDULER = p.SCHEDULER, walltime = "240:00:00", queue = p.QUEUE, nodes = 1, ppn = 4, memory = "15gb", script = "/GATK_variantFiltrationNoMask_DRAW.sh", args_list = [p.BWA_RESULTS, sample, p.TEMP_DIR, p.MMQ, p.GENOME, p.DBSNP, p.TARGET_FLANKS_FILE, p.GATK_THREADS, p.MBQ])
    job_status(jobname = 'GATK_variantFiltrationNoMask', resultspath = p.BWA_RESULTS, SAMPLE = sample,  outputfilename = sample + "/vcf/" +  sample + "_snpcal_unifiedGenotyper_mbq_" + p.MBQ + "_mmq_" + p.MMQ +"_filtered_nomask.vcf", FLAG_PATH = p.FLAG_PATH)
    return


def GATK_variantEval(sample, GATK_variantEval_flag):
    '''GATK_variant_eval.

    input:
        snpcal_unifiedGenotyper_mbq_${9}_mmq_${4}_filtered_nomask.vcf
    output:
        snpcal_unifiedGenotyper_mbq_${9}_mmq_${4}_filtered_nomask.vcf.eval
    citation:
        McKenna A, Hanna M, Banks E, Sivachenko A, Cibulskis K, Kernytsky A, Garimella K, Altshuler D, Gabriel S, Daly M, DePristo MA (2010). The Genome Analysis Toolkit: a MapReduce framework for analyzing next-generation DNA sequencing data. Genome Res. 20:1297-303.
    link: GATK_variant_discovery
        http://www.broadinstitute.org/gatk/
    parameters from parameters file:
        BWA_RESULTS:

        SAMPLE:

        TEMP_DIR:

        MMQ:

        GENOME:

        DBSNP:

        TARGET_FLANKS_FILE:

        GATK_THREADS:

        MBQ:

        '''

    spawn_job(jobname = 'GATK_variantEval', SAMPLE = sample, LOG_PATH = p.LOG_PATH, RESULTS_EMAIL = p.RESULTS_EMAIL, SCHEDULER = p.SCHEDULER, walltime = "240:00:00", queue = p.QUEUE, nodes = 1, ppn = 4, memory = "10gb", script = "/GATK_variantEval_DRAW.sh", args_list = [p.BWA_RESULTS, sample, p.TEMP_DIR, p.MMQ, p.GENOME, p.DBSNP, p.TARGET_FLANKS_FILE, p.GATK_THREADS, p.MBQ])
    job_status(jobname = 'GATK_variantEval', resultspath = p.BWA_RESULTS, SAMPLE = sample,  outputfilename = sample + "/vcf/" +  sample + "_snpcal_unifiedGenotyper_mbq_" + p.MBQ + "_mmq_" + p.MMQ +"_filtered_nomask.vcf.eval", FLAG_PATH = p.FLAG_PATH)
    return

def GATK_snpcal_haplotyperCaller(sample, GATK_snpcal_haplotyperCaller_flag):
    '''GATK_variant_discovery with HaplotypeCaller.

    input:
        recal.FULL.bam
    output:
        allbases.snps.indels.g.vcf
    citation:
        McKenna A, Hanna M, Banks E, Sivachenko A, Cibulskis K, Kernytsky A, Garimella K, Altshuler D, Gabriel S, Daly M, DePristo MA (2010). The Genome Analysis Toolkit: a MapReduce framework for analyzing next-generation DNA sequencing data. Genome Res. 20:1297-303.
    link: GATK_variant_discovery
        http://www.broadinstitute.org/gatk/
    parameters from parameters file:
        BWA_RESULTS:

        SAMPLE:

        TEMP_DIR:

        MMQ:

        GENOME:

        DBSNP:

        TARGET_FLANKS_FILE:

        GATK_THREADS:

        MBQ:

        '''

    spawn_job(jobname = 'GATK_snpcal_haplotyperCaller', SAMPLE = sample, LOG_PATH = p.LOG_PATH, RESULTS_EMAIL = p.RESULTS_EMAIL, SCHEDULER = p.SCHEDULER, walltime = "240:00:00", queue = p.QUEUE, nodes = 1, ppn = 4, memory = "15gb", script = "/GATK_snpcal_haplotyperCaller_DRAW.sh", args_list = [p.BWA_RESULTS, sample, p.TEMP_DIR, p.MMQ, p.GENOME, p.DBSNP, p.TARGET_FLANKS_FILE, p.GATK_THREADS, p.MBQ])
    job_status(jobname = 'GATK_snpcal_haplotyperCaller', resultspath = p.BWA_RESULTS, SAMPLE = sample,  outputfilename = sample + "/vcf/" +  sample  +".allbases.snps.indels.g.vcf", FLAG_PATH = p.FLAG_PATH)
    return



if __name__ == '__main__':
    GATK_snpcal_unifiedGenotyper(sample, GATK_snpcal_unifiedGenotyper_flag)
    GATK_variantFiltrationNoMask(smaple, GATK_variantFiltrationNoMask_flag)
    GATK_variantEval(sample, GATK_variantEval_flag)
    GATK_snpcal_haplotyperCaller(sample, GATK_snpcal_haplotyperCaller_flag)
    sys.exit(0)
